import threading
from time import sleep
from math import sin, cos, pi

class Generator:
  """ This class generates data.

  In our example, each generated data-item is a tuple with three
elements.
  """
  def __init__(self, dataSet):
    """ dataSet: DataSet
    """
    self.running = 0         # generate data iff true
    self.dataSet = dataSet   # DataSet object to which data-items are added

  def addDataItems(self):
    self.running = 1

    # Generate 100 data items:
    for i in range(100):
      t = i/25.
      x = sin(2*pi*t)/5
      y = cos(2*pi*t)/5

      # Build a data-item with the values, and send them to the plotter:
      self.sendPlotter( {'t':t, 'x':x, 'y':y} )

      sleep(0.1)  # pause to make it look as if some real calculation is
                  # happening (to emulate a real simulator at work)

      # running will be set to 0 by stop() called from another thread
      if not self.running:
        break

  def stop(self):
    self.running = 0


  def sendPlotter(self, names_values_dict):
    """ The keys in the dictionnary "names_values_dict" are matched with 
    the entries in "self.dataSet.titles" to know what entry correspond 
    to what index in the data-item.
    """
    titles = self.dataSet.titles  # ordered titles
    temp_data = [None]*len(titles)
    for name in names_values_dict.keys():
      try:
        i = titles.index(name)
      except ValueError:
        pass  # THIS SHOULDN'T HAPPEND
      temp_data[i] = names_values_dict[name]

    # data-item ordered : add it to dataSet
    # (this will trigger updating the appropriate trajectory plots in the GUI)
    self.dataSet.append( temp_data )

class GeneratorThread(threading.Thread):

  def __init__(self, generatorObject):
    """ generatorObject is a Generator instance. """

    self.theThread = None
    self.generatorObject = generatorObject

  def start(self):
    """ Start generating data-items. """

    # if (has been stopped) or (is finished)...
    if self.theThread == None or not(self.theThread.isAlive()):
      # Empty the DataSet
      self.generatorObject.dataSet.clear()
      # Create the thread
      self.theThread = \
       threading.Thread(target=self.generatorObject.addDataItems)
      # Start the thread
      self.theThread.start()

  def stop(self):
    """ Stop data-items from being generated. """

    self.generatorObject.stop()  # stop the generator
    if self.theThread != None:
      self.theThread.join()  # wait until the thread terminates
      self.theThread = None

def startResume(parentApp, model):
 # currently, this only (re-)starts, no resume
 if "data" in dir(model):
  model.generator = Generator(model.data)
  model.generatorThread = GeneratorThread(model.generator)
  model.generatorThread.start()
 else:
  print "\nWarning: need to set up data element (in plot window) first\n"

